# lab2_SDT_Graphs.R
#
# R script file for defining two plotting
# functions for making signal detection
# plots from the data in the *_grf file
# produced by RscorePlus:
#
# plot1() plots a probability ROC and
#	a z-score ROC in a single figure;
#
# plot2() plots the best-fitting Gaussian
#	signal detection model.
#
# 8 February 2014

# Edit below to put in your own name
myName = "Your_Name"

# read the RscorePlus graphic file (*_grf) into a data frame
fn <- file.choose()					# choose the file name
df <- read.delim(fn, header=TRUE, check.names=TRUE)

# check that the input file is an RscorePlus graphics file
doGrf <- as.logical(grepl("_grf", fn))
if (doGrf) {
	model.name = paste(df$Model[1], "Model")	# name of model

	plot1 <- function() {
		# plot ROC curve for without eyes in red and with eyes in blue
		par(mfrow = c(1,2))	# two graphs per figure
		par(pty = "s")		# make plots square

		# probability plot
		plot(Obs_pYs1 ~ Obs_pYs0,
			data = df,
			pch = 21, col = "red",
			xlim = c(0, 1),
			ylim = c(0, 1),
			xlab = "False Alarm Rate",
			ylab = "Hit Rate",
			main = paste(myName, paste(model.name, ": Probability ROC", sep = ""), sep="\n"))
		points(Obs_pYs2 ~ Obs_pYs0, data=df, pch=19, col="blue")
		lines(Pred_pYs1 ~ Pred_pYs0, data=df, col="red", lwd = 2)
		lines(Pred_pYs2 ~ Pred_pYs0, data=df, col="blue", lwd = 2)
		text(.8, .6, "Without\nEyes", col="red", cex=0.75)
		text(.2, .9, "With\nEyes", col="blue", cex=0.75)
		abline(0,1)

		# z-score plot
		plot(Obs_zYs1 ~ Obs_zYs0, data=df, pch=21, col="red",
			xlim=c(-2.5, 2.5),
			ylim=c(-2.5, 2.5),
			xlab="z-score of False Alarm Rate",
			ylab="z-score of Hit Rate",
			main = paste(myName, paste(model.name, ": Z-Score ROC", sep = ""), sep="\n"))
		points(Obs_zYs2 ~ Obs_zYs0, data=df, pch=19, col="blue")
		lines(Pred_zYs1 ~ Pred_zYs0, data=df, col="red", lwd=2)
		lines(Pred_zYs2 ~ Pred_zYs0, data=df, col="blue", lwd=2)
		text(1.4, .6, "Without\nEyes", col="red", cex=0.75)
		text(-1.7, 1, "With\nEyes", col="blue", cex=0.75)
		abline(0,1)

		par(pty = "m")		# restore the normal size of plots
		par(mfrow = c(1,1))	# restore the single plot per window mode
	} # end of plot1()

	plot2 <- function() {
		model.name = paste(df$Model[1], "Detection Model")
		# check that the model is Gaussian
		doPlot <- pmatch("Gaussian", df$Model, nomatch=FALSE)

		if(doPlot) {
			# plot distribution for without eyes in red
			# and with eyes in blue

			# ###########################################################
			# Figure out how many criteria and how many signal conditions 
			nsig <- df$nsig[1]	# number of signals
			nrat <- df$nrat[1]	# number of ratings
			ncrt <- nrat - 1	# number of criteria

			# Plot the signal detection model
			# find the maximum density value for setting y-axis
			# find the s0 probability density column: save in ind0
			# find the last signal column: save in ind1
			ind0 <- match("Pred_Density0", names(df))
			ind1 <- ind0 + nsig - 1
			max.density <- 1.3 * max(df[, ind0:ind1], na.rm = TRUE)
			par(pty = "m")
			# Set up the plot
			plot(df[, ind0] ~ Xc, data = df, type = "n",
				lwd = 2, col = "black",
				xlim = c(min(df$Xc, na.rm = TRUE), max(df$Xc, na.rm = TRUE)),
				ylim = c(-0.03, max.density),
				xlab = "Decision Axis (standard deviations of S0)",
				ylab = "Probability Density",
				main = paste(myName, model.name, sep = "\n"))
			# plot the probability density functions (s0, s1 ...)
			colors <- c("black", "red", "blue")
			for (i in 0:(nsig-1)){
				lines(df[, ind0+i] ~ Xc, data = df,
				type = "l",
				lwd = 2,
				col = colors[i+1])
			}
			abline(h = 0, col = "gray", lty = "solid")
			# extract the criterion coordinates
			crit <- subset(df, Criteria != is.na(Criteria))[c("Xc", "Xc_se", "Criteria")]
			xc <- df$Xc[1:ncrt]	
			# plot the decision criterion markers
			for (i in 1:ncrt){
				segments(crit$Xc[(2*i)-1], crit$Criteria[(2*i)-1],
					crit$Xc[2*i], crit$Criteria[2*i], col="darkgreen")}
			# plot the standard error of each criterion marker
			for (i in 1:ncrt){
				segments(crit$Xc[2*i]-crit$Xc_se[2*i], crit$Criteria[2*i],
				crit$Xc[2*i]+crit$Xc_se[2*i], crit$Criteria[2*i], col = "darkgreen")}
			# label the response categories
			xc.lab <- rep(0, nrat)
			xc.lab[2:ncrt] <- xc[1:ncrt-1] + diff(xc)/2
			xc.lab[1] <- xc[1] - (diff(xc)/2)[1]
			xc.lab[nrat] <- xc[ncrt] + (diff(xc)/2)[ncrt-1]
			text(xc.lab, -0.025, labels = 1:nrat, cex = 0.7, col="darkgreen")

			# label the distributions
			text(-1, .35, "new\nfaces", cex=0.7, col=colors[1])
			text(1.8, .23, "old\nno eyes", cex=0.7, col=colors[2])
			text(2.9, .29, "old\neyes", cex=0.7, col=colors[3])
		} else
		{
			print("Function plot2() can only plot Gaussian models.")
			print("Select the Gaussian grf file as input.")
		} # end of doPlot if statement
	} # end of plot2()
} else {
	print("The input file must be a grf file from RscorePlus.")
} # end of doGrf if file test at top

plot1()
